#ifndef LIBR2C2_BLOCK_H_
#define LIBR2C2_BLOCK_H_

#include <list>
#include <set>
#include "track.h"
#include "trackchain.h"

namespace R2C2 {

class Layout;
class Route;
class TrackCircuit;
class TrackIter;
class Train;

class Block: public TrackChain
{
public:
	struct Endpoint
	{
		Track *track;
		unsigned track_ep;
		Block *link;

		Endpoint(Track *, unsigned);

		TrackIter track_iter() const;
	};

	sigc::signal<void, Train *> signal_reserved;

private:
	unsigned id;
	unsigned sensor_addr;
	unsigned turnout_addr;
	bool conflict;
	TrackCircuit *sensor;
	std::vector<Endpoint> endpoints;
	Train *train;

public:
	Block(Layout &, Track &);
	~Block();

	virtual void set_name(const std::string &);

private:
	virtual void on_track_added(Track &);
	virtual Validity check_validity(Track &) const;

public:
	unsigned get_id() const { return id; }
	unsigned get_sensor_address() const { return sensor_addr; }
	unsigned get_turnout_address() const { return turnout_addr; }
	TrackCircuit *get_sensor() const { return sensor; }
	const std::vector<Endpoint> &get_endpoints() const { return endpoints; }
	const Endpoint &get_endpoint(unsigned) const;
	int get_endpoint_by_link(Block &) const;
	float get_path_length(unsigned, const Route * = 0) const;
	void check_link(Block &);
	void break_link(Block &);
	Block *get_link(unsigned) const;
	bool reserve(Train *);
	Train *get_train() const { return train; }
private:
	void determine_id();

public:
	virtual Msp::DataFile::Statement save_reference() const;
};

} // namespace R2C2

#endif
