#include "block.h"
#include "blockiter.h"
#include "route.h"
#include "trackiter.h"

using namespace std;
using namespace Msp;

namespace R2C2 {

BlockIter::BlockIter():
	_block(0),
	_entry(0)
{ }

BlockIter::BlockIter(Block *b, unsigned e):
	_block(b),
	_entry(b ? e : 0)
{
	if(_block && _entry>_block->get_endpoints().size())
		throw out_of_range("BlockIter::BlockIter");
}

TrackIter BlockIter::track_iter() const
{
	if(!_block)
		return TrackIter();

	return _block->get_endpoint(_entry).track_iter();
}

const Block::Endpoint &BlockIter::endpoint() const
{
	if(!_block)
		throw logic_error("null block");

	return _block->get_endpoint(_entry);
}

int BlockIter::get_exit(const Route *route) const
{
	const vector<Block::Endpoint> &eps = _block->get_endpoints();
	if(_block->get_turnout_address())
	{
		/* The endpoints of a turnout block exactly correspond to the endpoints
		of the track. */
		TrackIter t_iter = track_iter();
		unsigned path = (route ? route->get_path(*t_iter) : t_iter->get_active_path());
		return t_iter.reverse(path).entry();
	}
	else if(eps.size()==2)
		return 1-_entry;
	else
		return -1;
}

BlockIter BlockIter::next(const Route *route) const
{
	if(!_block)
		return BlockIter();

	int exit = get_exit(route);
	if(exit<0)
		return BlockIter();

	BlockIter result;
	result._block = _block->get_link(exit);
	result._entry = (result._block ? result._block->get_endpoint_by_link(*_block) : 0);

	return result;
}

BlockIter BlockIter::reverse(const Route *route) const
{
	if(!_block)
		return BlockIter();

	int exit = get_exit(route);
	if(exit<0)
		return BlockIter();

	return BlockIter(_block, exit);
}

BlockIter BlockIter::flip() const
{
	if(!_block)
		return BlockIter();

	BlockIter result;
	result._block = _block->get_link(_entry);
	result._entry = (result._block ? result._block->get_endpoint_by_link(*_block) : 0);

	return result;
}

Block &BlockIter::operator*() const
{
	if(!_block)
		throw logic_error("null block");

	return *_block;
}

bool BlockIter::operator==(const BlockIter &other) const
{
	return _block==other._block && _entry==other._entry;
}

} // namespace R2C2
