#include <msp/strings/format.h>
#include <msp/strings/regex.h>
#include "block.h"
#include "layout.h"
#include "track.h"
#include "zone.h"

using namespace std;
using namespace Msp;

namespace R2C2 {

Zone::Zone(Layout &l):
	TrackChain(l),
	number(0)
{
	layout.add(*this);
}

Zone::~Zone()
{
	layout.remove(*this);
}

void Zone::set_name(const string &n)
{
	Regex r_name("(.+) ([^ ]+) ([1-9][0-9]*)");
	RegMatch m = r_name.match(n);
	if(!m)
		throw invalid_argument("Zone::set_name");

	set_name(m[1].str, m[2].str, lexical_cast<unsigned>(m[3].str));
}

void Zone::set_name(const string &g, const string &q, unsigned n)
{
	group = g;
	qualifier = q;
	number = n;

	update_name();
}

void Zone::update_name()
{
	string full_name = group;
	if(!qualifier.empty())
	{
		full_name += ' ';
		full_name += qualifier;
	}
	if(number)
		full_name += format(" %d", number);
	TrackChain::set_name(full_name);
}

void Zone::save(list<DataFile::Statement> &st) const
{
	st.push_back((DataFile::Statement("group"), group));
	if(!qualifier.empty())
		st.push_back((DataFile::Statement("qualifier"), qualifier));
	if(number)
		st.push_back((DataFile::Statement("number"), number));

	unsigned last_block = 0;
	TrackIter iter = ends[0].reverse();
	while(iter && tracks.count(iter.track()))
	{
		unsigned block_id = iter->get_block().get_id();
		if(block_id!=last_block)
		{
			st.push_back((DataFile::Statement("block"), block_id));
			last_block = block_id;
		}

		iter = next_iter(iter);
	}
}

DataFile::Statement Zone::save_reference() const
{
	return (DataFile::Statement("zone"), group, number);
}


Zone::Loader::Loader(Zone &z):
	DataFile::ObjectLoader<Zone>(z)
{
	add("block",     &Loader::block);
	add("group",     &Zone::group);
	add("number",    &Zone::number);
	add("qualifier", &Zone::qualifier);
}

void Zone::Loader::finish()
{
	obj.update_name();
}

void Zone::Loader::block(unsigned b)
{
	Block &blk = obj.layout.get_block(b);
	obj.add_tracks(blk.get_tracks());
}

} // namespace R2C2
